using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace CharacterDemo_39677
{
    public class MeasureStringTextBoxCellModel : GridTextBoxCellModel
    {
        public MeasureStringTextBoxCellModel(GridModel grid)
            : base(grid)
        {

        }

        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new MeasureStringTextBoxCellRenderer(control, this);
        }

        protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds queryBounds)
        {
            // ** default
            // return base.OnQueryPrefferedClientSize(g, rowIndex, colIndex, style, queryBounds);

            //** Method1
            int stringWidth = MeasureStringTextBoxCellRenderer.MeasureDisplayStringWidth_method1(g, style.Text, style.Font.GdipFont);
            return new Size(stringWidth, base.OnQueryPrefferedClientSize(g, rowIndex, colIndex, style, queryBounds).Height);

            //** Method2
            //int stringWidth = MeasureStringTextBoxCellRenderer.MeasureDisplayStringWidth_method2(g, style.Text, style.Font.GdipFont);
            //return new Size(stringWidth, base.OnQueryPrefferedClientSize(g, rowIndex, colIndex, style, queryBounds).Height);
        }
    }

    public class MeasureStringTextBoxCellRenderer : GridTextBoxCellRenderer
    {
        public MeasureStringTextBoxCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
        }

            protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
            {
                if (Grid.CurrentCell.HasCurrentCellAt(rowIndex, colIndex) && CurrentCell.IsEditing && !Grid.PrintingMode)
                {
                    base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);
                }
                else
                {
                    string displayText = style.FormattedText;
                    if (displayText.Length > 0)
                    {
                        Font font = style.GdipFont;
                        Rectangle textRectangle = RemoveMargins(clientRectangle, style);
                        if (style.HorizontalAlignment == GridHorizontalAlignment.Right)
                        {
                            int stringWidth = MeasureDisplayStringWidth_method1(g, displayText, font);

                            stringWidth = Math.Min(stringWidth, textRectangle.Width);

                            textRectangle = new Rectangle(textRectangle.Right - stringWidth, textRectangle.Y, stringWidth, textRectangle.Height);

                            style.HorizontalAlignment = GridHorizontalAlignment.Left;
                        }
                        Color textColor = Grid.PrintingMode && Grid.Model.Properties.BlackWhite ? Color.Black : style.TextColor;
                        using (SolidBrush br = new SolidBrush(textColor))
                        {
                          g.DrawString(displayText, font, br, textRectangle, StringFormat.GenericTypographic);
                        }
                    }
                }
            }


        //code from http://www.codeproject.com/cs/media/measurestring.asp
        static public int MeasureDisplayStringWidth_method2(Graphics graphics, string text, Font font)
        {
            RectangleF rect = RectangleF.Empty;

            using (StringFormat format = new System.Drawing.StringFormat())
            {
                rect = new System.Drawing.RectangleF(0, 0, 1000, 1000);
                CharacterRange[] ranges = { new System.Drawing.CharacterRange(0, text.Length) };
                Region[] regions = new System.Drawing.Region[1];
                format.SetMeasurableCharacterRanges(ranges);
                regions = graphics.MeasureCharacterRanges(text, font, rect, format);
                rect = regions[0].GetBounds(graphics);
            }

            return (int)(rect.Right + 1.0f);
        }

        static public int MeasureDisplayStringWidth_method1(Graphics graphics, string text,
                                            Font font)
        {
            const int width = 32;

            System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(width, 1,
                                                                        graphics);
            System.Drawing.SizeF size = graphics.MeasureString(text, font);
            System.Drawing.Graphics anagra = System.Drawing.Graphics.FromImage(bitmap);

            int measured_width = (int)size.Width;

            if (anagra != null)
            {
                anagra.Clear(Color.White);
                anagra.DrawString(text + "|", font, Brushes.Black,
                                   width - measured_width, -font.Height / 2);

                for (int i = width - 1; i >= 0; i--)
                {
                    measured_width--;
                    if (bitmap.GetPixel(i, 0).R != 255)    // found a non-white pixel ?
                        break;
                }
            }

            return measured_width;
        }

    }
}
